@@ -29,7 +29,7 @@ static int is_threaded = -1;
  * libraries.
  */
 #ifdef __GNUC__
-#ifdef linux
+#if defined(linux) || defined(__GLIBC__)
 #if (__GNUC__ == 3 && __GNUC_MINOR__ >= 3) || (__GNUC__ > 3)
 extern int pthread_mutexattr_init(pthread_mutexattr_t *attr)
            __attribute((weak));
@@ -459,7 +459,6 @@ gamin_data_new(void)
         if ((pthread_mutexattr_init != NULL) &&
 	    (pthread_mutexattr_settype != NULL) &&
 	    (pthread_mutex_init != NULL) &&
-	    (pthread_mutexattr_destroy != NULL) &&
 	    (pthread_mutex_lock != NULL) &&
 	    (pthread_mutex_unlock != NULL) &&
 	    (pthread_mutexattr_destroy != NULL)) {
@@ -471,7 +470,7 @@ gamin_data_new(void)
     }
     if (is_threaded > 0) {
 	pthread_mutexattr_init(&attr);
-#ifdef __GLIBC__
+#if defined(linux) || defined(PTHREAD_MUTEX_RECURSIVE_NP)
 	pthread_mutexattr_settype(&attr, PTHREAD_MUTEX_RECURSIVE_NP);
 #else
 	pthread_mutexattr_settype(&attr, PTHREAD_MUTEX_RECURSIVE);
@@ -540,6 +539,7 @@ gamin_data_reset(GAMDataPtr conn, GAMReqDataPtr **requests)
     conn->reqno = 1;
     conn->restarted = 1;
     conn->evn_ready = 0;
+    conn->evn_read = 0;
     return(conn->req_nr);
 }
 
