@@ -1,4 +1,4 @@
-#include "config.h"
+#include "server_config.h"
 #include <stdlib.h>
 #include <unistd.h>
 #include <errno.h>
@@ -6,11 +6,13 @@
 #include <sys/socket.h>
 #include <sys/stat.h>
 #include <sys/un.h>
+#include <sys/uio.h>
 #include "gam_error.h"
 #include "gam_connection.h"
 #include "gam_channel.h"
 #include "gam_protocol.h"
 
+/* #define CHANNEL_VERBOSE_DEBUGGING */
 /************************************************************************
  *									*
  *			Connection socket handling			*
@@ -24,11 +26,55 @@
  * to check the server credentials early on.
  */
 static gboolean
-gam_client_conn_send_cred(GIOChannel * source, int fd)
+gam_client_conn_send_cred(int fd)
 {
     char data[2] = { 0, 0 };
+    int written;
+#if defined(HAVE_CMSGCRED) && !defined(LOCAL_CREDS)
+    struct {
+	    struct cmsghdr hdr;
+	    struct cmsgcred cred;
+    } cmsg;
+    struct iovec iov;
+    struct msghdr msg;
 
-    return(gam_client_conn_write(source, fd, &data[0], 1));
+    iov.iov_base = &data[0];
+    iov.iov_len = 1;
+
+    memset (&msg, 0, sizeof (msg));
+    msg.msg_iov = &iov;
+    msg.msg_iovlen = 1;
+
+    msg.msg_control = &cmsg;
+    msg.msg_controllen = sizeof (cmsg);
+    memset (&cmsg, 0, sizeof (cmsg));
+    cmsg.hdr.cmsg_len = sizeof (cmsg);
+    cmsg.hdr.cmsg_level = SOL_SOCKET;
+    cmsg.hdr.cmsg_type = SCM_CREDS;
+#endif
+
+retry:
+#if defined(HAVE_CMSGCRED) && !defined(LOCAL_CREDS)
+    written = sendmsg(fd, &msg, 0);
+#else
+    written = write(fd, &data[0], 1);
+#endif
+    if (written < 0) {
+        if (errno == EINTR)
+            goto retry;
+	gam_error(DEBUG_INFO,
+		  "Failed to write credential bytes to socket %d\n", fd);
+	return (-1);
+    }
+    if (written != 1) {
+	gam_error(DEBUG_INFO, "Wrote %d credential bytes to socket %d\n",
+		  written, fd);
+	return (-1);
+    }
+#ifdef CHANNEL_VERBOSE_DEBUGGING
+    GAM_DEBUG(DEBUG_INFO, "Wrote credential bytes to socket %d\n", fd);
+#endif
+    return (written);
 }
 
 /**
@@ -49,8 +95,10 @@ gam_client_conn_check_cred(GIOChannel * source, int fd,
     gid_t c_gid;
 
 #ifdef HAVE_CMSGCRED
-    char cmsgmem[CMSG_SPACE(sizeof(struct cmsgcred))];
-    struct cmsghdr *cmsg = (struct cmsghdr *) cmsgmem;
+    struct {
+	    struct cmsghdr hdr;
+	    struct cmsgcred cred;
+    } cmsg;
 #endif
 
     s_uid = getuid();
@@ -75,9 +123,9 @@ gam_client_conn_check_cred(GIOChannel * source, int fd,
     msg.msg_iovlen = 1;
 
 #ifdef HAVE_CMSGCRED
-    memset(cmsgmem, 0, sizeof(cmsgmem));
-    msg.msg_control = cmsgmem;
-    msg.msg_controllen = sizeof(cmsgmem);
+    memset(&cmsg, 0, sizeof(cmsg));
+    msg.msg_control = &cmsg;
+    msg.msg_controllen = sizeof(cmsg);
 #endif
 
   retry:
@@ -94,7 +142,7 @@ gam_client_conn_check_cred(GIOChannel * source, int fd,
         goto failed;
     }
 #ifdef HAVE_CMSGCRED
-    if (cmsg->cmsg_len < sizeof(cmsgmem) || cmsg->cmsg_type != SCM_CREDS) {
+    if (cmsg.hdr.cmsg_len < sizeof(cmsg) || cmsg.hdr.cmsg_type != SCM_CREDS) {
         GAM_DEBUG(DEBUG_INFO,
                   "Message from recvmsg() was not SCM_CREDS\n");
         goto failed;
@@ -106,7 +154,7 @@ gam_client_conn_check_cred(GIOChannel * source, int fd,
     {
 #ifdef SO_PEERCRED
         struct ucred cr;
-        int cr_len = sizeof(cr);
+        socklen_t cr_len = sizeof(cr);
 
         if (getsockopt(fd, SOL_SOCKET, SO_PEERCRED, &cr, &cr_len) ==
             0 && cr_len == sizeof(cr)) {
@@ -120,13 +168,9 @@ gam_client_conn_check_cred(GIOChannel * source, int fd,
             goto failed;
         }
 #elif defined(HAVE_CMSGCRED)
-        struct cmsgcred *cred;
-
-        cred = (struct cmsgcred *) CMSG_DATA(cmsg);
-
-        c_pid = cred->cmcred_pid;
-        c_uid = cred->cmcred_euid;
-        c_gid = cred->cmcred_groups[0];
+	c_pid = cmsg.cred.cmcred_pid;
+	c_uid = cmsg.cred.cmcred_euid;
+	c_gid = cmsg.cred.cmcred_groups[0];
 #else /* !SO_PEERCRED && !HAVE_CMSGCRED */
         GAM_DEBUG(DEBUG_INFO,
                   "Socket credentials not supported on this OS\n");
@@ -149,7 +193,7 @@ gam_client_conn_check_cred(GIOChannel * source, int fd,
         goto failed;
     }
 
-    if (!gam_client_conn_send_cred(source, fd)) {
+    if (!gam_client_conn_send_cred(fd)) {
         GAM_DEBUG(DEBUG_INFO, "Failed to send credential byte to client\n");
         goto failed;
     }
@@ -226,7 +270,9 @@ retry:
         gam_client_conn_shutdown(source, conn);
         return (FALSE);
     }
+#ifdef CHANNEL_VERBOSE_DEBUGGING
     GAM_DEBUG(DEBUG_INFO, "read %d bytes from client\n", ret);
+#endif
 
     /* 
      * there is no garantee of alignment, that the request is complete
@@ -277,6 +323,8 @@ gam_incoming_conn_read(GIOChannel * source, GIOCondition condition,
     return (FALSE);
 }
 
+// #define GAM_CHANNEL_VERBOSE
+
 /************************************************************************
  *									*
  *			Path for the socket connection			*
@@ -305,6 +353,7 @@ gam_get_socket_path(const char *session)
         gam_client_id = g_getenv("GAM_CLIENT_ID");
         if (gam_client_id == NULL) {
             GAM_DEBUG(DEBUG_INFO, "Error getting GAM_CLIENT_ID\n");
+	    gam_client_id = "";
         }
     } else {
         gam_client_id = session;
@@ -455,7 +504,9 @@ create:
 	    return(FALSE);
 	}
     }
+#ifdef GAM_CHANNEL_VERBOSE
     GAM_DEBUG(DEBUG_INFO, "Removed %s\n", dir);
+#endif
     tries++;
     if (tries < 5)
         goto create;
@@ -721,7 +772,7 @@ gam_client_create(GIOChannel * server)
     GIOChannel *socket = NULL;
     int sock;
     int client = -1;
-    int client_addrlen;
+    socklen_t client_addrlen;
     struct sockaddr client_addr;
 
     sock = g_io_channel_unix_get_fd(server);
@@ -759,6 +810,7 @@ gam_client_conn_write(GIOChannel * source, int fd, gpointer data,
                       size_t len)
 {
     int written;
+    int remaining;
 
     /**
      * Todo: check if write will block, or use non-blocking options
@@ -769,19 +821,25 @@ gam_client_conn_write(GIOChannel * source, int fd, gpointer data,
     if (fd < 0)
         return (FALSE);
 
-retry:
-    written = write(fd, data, len);
-    if (written < 0) {
-        if (errno == EINTR)
-            goto retry;
-        GAM_DEBUG(DEBUG_INFO, "Failed to write bytes to socket %d\n", fd);
-        return (FALSE);
-    }
-    if (written != (int) len) {
-        GAM_DEBUG(DEBUG_INFO, "Wrote only %d bytes to socket %d\n",
-                  written, fd);
-        return (FALSE);
-    }
-    GAM_DEBUG(DEBUG_INFO, "Wrote %d bytes to socket %d\n", written, fd);
+    remaining = len;
+    do {
+	written = write(fd, data, remaining);
+	if (written < 0) {
+	    if (errno == EINTR)
+		continue;
+
+	    GAM_DEBUG(DEBUG_INFO,
+		      "%s: Failed to write bytes to socket %d: %s\n",
+		      __FUNCTION__, fd, strerror (errno));
+	    return (FALSE);
+	}
+
+	data += written;
+	remaining -= written;
+    } while (remaining > 0);
+
+#ifdef CHANNEL_VERBOSE_DEBUGGING
+    GAM_DEBUG(DEBUG_INFO, "Wrote %d bytes to socket %d\n", len, fd);
+#endif
     return (TRUE);
 }
