@@ -17,21 +17,12 @@
  */
 
 
-#include <config.h>
+#include "server_config.h"
 #include <string.h>
 #include <glib.h>
 #include "gam_event.h"
 #include "gam_node.h"
-
-/**
- * @defgroup GamNode GamNode
- * @ingroup Daemon
- * @brief GamNode API.
- *
- * A node represents a single file or directory.
- * 
- * @{
- */
+#include "gam_error.h"
 
 /**
  * Create a new node
@@ -45,7 +36,6 @@ GamNode *
 gam_node_new(const char *path, GamSubscription * sub, gboolean is_dir)
 {
     GamNode *node;
-
     node = g_new0(GamNode, 1);
 
     node->path = g_strdup(path);
@@ -55,10 +45,12 @@ gam_node_new(const char *path, GamSubscription * sub, gboolean is_dir)
         node->subs = NULL;
 
     node->is_dir = is_dir;
-    node->data = NULL;
-    node->data_destroy = NULL;
     node->flags = 0;
+	node->checks = 0;
 
+    node->poll_time = gam_fs_get_poll_timeout (path);
+    node->mon_type = gam_fs_get_mon_type (path);
+	GAM_DEBUG(DEBUG_INFO, "g_n_n: node for %s using %s with poll timeout of %d\n", path, node->mon_type == GFS_MT_KERNEL ? "kernel" : "poll", node->poll_time);
     return node;
 }
 
@@ -70,16 +62,10 @@ gam_node_new(const char *path, GamSubscription * sub, gboolean is_dir)
 void
 gam_node_free(GamNode * node)
 {
-    g_return_if_fail(node != NULL);
-
+    g_assert(node != NULL);
     g_assert(node->subs == NULL);
 
-    if (node->data_destroy && node->data)
-        (*node->data_destroy) (node->data);
-
     g_free(node->path);
-    g_list_free(node->subs);
-    node->subs = NULL;
     g_free(node);
 }
 
@@ -87,15 +73,15 @@ gam_node_free(GamNode * node)
  * Retrieves the parent of a given node
  *
  * @param node the node
- * @returns the parent, or NULL
+ * @returns the parent, or NULL if node has no parent
  */
 GamNode *
 gam_node_parent(GamNode * node)
 {
     GamNode *ret = NULL;
 
-    if (!node)
-        return(NULL);
+    g_assert(node);
+
     if (node->node && node->node->parent)
         ret = (GamNode *) node->node->parent->data;
 
@@ -103,7 +89,7 @@ gam_node_parent(GamNode * node)
 }
 
 /**
- * Checks whether a node is a directory or not
+ * Returns whether a node is a directory or not
  *
  * @param node the node
  * @returns TRUE if the node is a directory, FALSE otherwise
@@ -111,13 +97,12 @@ gam_node_parent(GamNode * node)
 gboolean
 gam_node_is_dir(GamNode * node)
 {
-    if (node == NULL)
-        return(FALSE);
+    g_assert(node);
     return(node->is_dir);
 }
 
 /**
- * Sets whether a node is a directory or not
+ * Sets whether a node is a directory
  *
  * @param node the node
  * @param is_dir whether the node is a directory
@@ -125,54 +110,50 @@ gam_node_is_dir(GamNode * node)
 void
 gam_node_set_is_dir(GamNode * node, gboolean is_dir)
 {
-    if (!node)
-        return;
+    g_assert(node);
     node->is_dir = is_dir;
 }
 
 /**
- * Gets the path a given node represents
+ * Returns the path associated with a node
  *
  * @param node the node
- * @returns The path.  It should not be freed.
+ * @returns the path.  The caller must keep a reference to node until
+ * it has finished with the string.  If it must keep it longer, it
+ * should makes its own copy.  The returned string must not be freed.
  */
 G_CONST_RETURN char *
 gam_node_get_path(GamNode * node)
 {
-    if (!node)
-        return(NULL);
+    g_assert(node);
     return node->path;
 }
 
 /**
- * Returns a list of subscriptions attached to the node
+ * Returns the list of subscriptions to a node
  *
  * @param node the node
- * @returns a list of #GamSubscription, or NULL
+ * @returns the list of #GamSubscription to the node
  */
 GList *
 gam_node_get_subscriptions(GamNode * node)
 {
-    if (!node)
-        return(NULL);
+    g_assert(node);
     return node->subs;
 }
 
 /**
- * gam_node_has_dir_subscriptions:
- * @node: the node
+ * Returns whether a node has any directory subscriptions
  *
- * Allow to find if a node has directory subscriptions
- *
- * Returns TRUE if yes and FALSE otherwise
+ * @param node the node
+ * @returns TRUE if the node has directory subscriptions, FALSE otherwise
  */
 gboolean
 gam_node_has_dir_subscriptions(GamNode * node)
 {
     GList *s;
 
-    if (!node)
-        return(FALSE);
+    g_assert (node);
     if (!(node->is_dir))
         return(FALSE);
     for (s = node->subs;s != NULL;s = s->next) {
@@ -187,15 +168,16 @@ gam_node_has_dir_subscriptions(GamNode * node)
  *
  * @param node the node
  * @param sub the subscription
+ * @returns TRUE on success, FALSE otherwise
  */
 gboolean
 gam_node_add_subscription(GamNode * node, GamSubscription * sub)
 {
+    g_assert(node);
+    g_assert(sub);
+    g_assert(!g_list_find(node->subs, sub));
 
-    if (!node)
-        return(FALSE);
-    if (!g_list_find(node->subs, sub))
-        node->subs = g_list_prepend(node->subs, sub);
+    node->subs = g_list_prepend(node->subs, sub);
 
     return TRUE;
 }
@@ -210,47 +192,16 @@ gam_node_add_subscription(GamNode * node, GamSubscription * sub)
 gboolean
 gam_node_remove_subscription(GamNode * node, GamSubscription * sub)
 {
-    if (!node)
-        return(FALSE);
+    g_assert(node);
+    g_assert(g_list_find (node->subs, sub));
+
     node->subs = g_list_remove_all(node->subs, sub);
 
     return TRUE;
 }
 
 /**
- * Attaches some arbitrary data to the node
- *
- * @param node the node
- * @param data a pointer to some data
- * @param destroy a function to destroy the data when the node is freed, or NULL
- */
-void
-gam_node_set_data(GamNode * node, gpointer data, GDestroyNotify destroy)
-{
-    if (node == NULL)
-        return;
-
-    node->data = data;
-    node->data_destroy = destroy;
-
-}
-
-/**
- * Retrieves the data attached to the node
- *
- * @param node the node
- * @returns the data, or NULL
- */
-gpointer
-gam_node_get_data(GamNode * node)
-{
-    if (node == NULL)
-        return(NULL);
-    return node->data;
-}
-
-/**
- * Sets the GNode associated with this node.  Should only be used by MdTree
+ * Sets the GNode associated with a node.  Should only be used by MdTree
  *
  * @param node the node
  * @param gnode the GNode
@@ -258,13 +209,12 @@ gam_node_get_data(GamNode * node)
 void
 gam_node_set_node(GamNode * node, GNode * gnode)
 {
-    if (node == NULL)
-        return;
+    g_assert(node);
     node->node = gnode;
 }
 
 /**
- * Gets the GNode associated with this node.  Should only be used by MdTree
+ * Gets the GNode associated with a node.  Should only be used by MdTree
  *
  * @param node the node
  * @returns the GNode
@@ -272,27 +222,27 @@ gam_node_set_node(GamNode * node, GNode * gnode)
 GNode *
 gam_node_get_node(GamNode * node)
 {
-    if (node == NULL)
-        return(NULL);
+    g_assert(node);
     return node->node;
 }
 
 /**
- * Sets a flag on the node
+ * Set a flag on a node
  *
  * @param node the node
- * @param flag the flag
+ * @param flag the flag to set
  */
 void
 gam_node_set_flag(GamNode * node, int flag)
 {
-    if (node == NULL)
-        return;
+    g_assert(node);
+    /* Make sure we set exactly one flag.  */
+    g_assert((flag & (flag - 1)) == 0);
     node->flags |= flag;
 }
 
 /**
- * Removes a flag on the node
+ * Clears a flag from a node
  *
  * @param node the node
  * @param flag the flag
@@ -300,24 +250,131 @@ gam_node_set_flag(GamNode * node, int flag)
 void
 gam_node_unset_flag(GamNode * node, int flag)
 {
-    if (node == NULL)
-        return;
+    g_assert(node);
+    /* Make sure we clear exactly one flag.  */
+    g_assert((flag & (flag - 1)) == 0);
     node->flags &= ~flag;
 }
 
 /**
- * Checks whether a node has a given flag
+ * Checks whether a flag is set on a node
  *
  * @param node the node
  * @param flag the flag
- * @returns TRUE if the node has the flag, FALSE otherwise
+ * @returns TRUE if the flag is set, FALSE otherwise
  */
 gboolean
 gam_node_has_flag(GamNode * node, int flag)
 {
-    if (node == NULL)
-        return(FALSE);
+    g_assert(node);
+    /* Check exactly one flag.  */
+    g_assert((flag & (flag - 1)) == 0);
     return node->flags & flag;
 }
 
+/**
+ * Set a pflag on a node
+ *
+ * @param node the node
+ * @param flag the flag to set
+ */
+void
+gam_node_set_pflag(GamNode * node, int flag)
+{
+    g_assert(node);
+    /* Make sure we set exactly one flag.  */
+    g_assert((flag & (flag - 1)) == 0);
+    node->pflags |= flag;
+}
+
+/**
+ * Clears a pflag from a node
+ *
+ * @param node the node
+ * @param flag the flag
+ */
+void
+gam_node_unset_pflag(GamNode * node, int flag)
+{
+    g_assert(node);
+    /* Make sure we clear exactly one flag.  */
+    g_assert((flag & (flag - 1)) == 0);
+    node->pflags &= ~flag;
+}
+
+/**
+ * Checks whether a pflag is set on a node
+ *
+ * @param node the node
+ * @param flag the flag
+ * @returns TRUE if the flag is set, FALSE otherwise
+ */
+gboolean
+gam_node_has_pflag(GamNode * node, int flag)
+{
+    g_assert(node);
+    /* Check exactly one flag.  */
+    g_assert((flag & (flag - 1)) == 0);
+    return node->pflags & flag;
+}
+
+/**
+ * Set the pflags on a node
+ *
+ * @param node the node
+ * @param flags the flags
+ */
+void
+gam_node_set_pflags(GamNode * node, int flags)
+{
+    g_assert(node);
+    node->pflags = flags;
+}
+
+
+/**
+ * Checks whether some pflags are set on a node
+ *
+ * @param node the node
+ * @param flags the flags
+ * @returns TRUE if the flag is set, FALSE otherwise
+ */
+gboolean
+gam_node_has_pflags(GamNode * node, int flags)
+{
+    g_assert(node);
+    return node->pflags & flags;
+}
+
+void
+gam_node_emit_event (GamNode *node, GaminEventType event)
+{
+	GList *l;
+	GamNode *parent;
+	GList *subs;
+	int is_dir_node = gam_node_is_dir(node);
+
+#ifdef VERBOSE_POLL
+	GAM_DEBUG(DEBUG_INFO, "Poll: emit events %d for %s\n", event, gam_node_get_path(node));
+#endif
+	subs = gam_node_get_subscriptions(node);
+
+	if (subs)
+		subs = g_list_copy(subs);
+
+	parent = gam_node_parent(node);
+	if (parent) {
+		GList *parent_subs = gam_node_get_subscriptions(parent);
+
+		for (l = parent_subs; l; l = l->next) {
+			if (!g_list_find(subs, l->data))
+				subs = g_list_prepend(subs, l->data);
+		}
+	}
+
+	gam_server_emit_event(gam_node_get_path(node), is_dir_node, event, subs, 0);
+
+	g_list_free(subs);
+}
+
 /** @} */
