@@ -18,7 +18,7 @@
  */
 
 
-#include <config.h>
+#include "server_config.h"
 #include <stdlib.h>
 #include <string.h>
 #include <glib.h>
@@ -198,6 +198,9 @@ gam_tree_add_at_path(GamTree * tree, const char *path, gboolean is_dir)
     if ((node = gam_tree_get_at_path(tree, path)) != NULL)
         return node;
 
+    if (g_file_test(path, G_FILE_TEST_EXISTS))
+        is_dir = g_file_test(path, G_FILE_TEST_IS_DIR);
+
     path_cpy = g_strdup(path);
 
     parent = NODE_DATA(tree->root);
